body { 
width:100% !important;
margin:0 !important;
padding:0 !important;
line-height: 1.4;
font-family: "Times New Roman", serif; color: #000; background: none; font-size: 12pt; }

/*Headings */
h1,h2,h3,h4,h5,h6 { font-family: Helvetica, Arial, sans-serif; }
h1{font-size:19pt;}
h2{font-size:17pt;}
h3{font-size:15pt;}
h4,h5,h6{font-size:12pt;}

h2.revision { font-size: 10pt; font-weight: normal; font-style: italic; text-align: right; }

pre, code { font: 10pt Courier, monospace; } 
blockquote { margin: 1.3em; padding: 1em;  font-size: 10pt; }
hr { background-color: #ccc; }

/* Images */
img { float: left; margin: 1em 1.5em 1.5em 0; }
a img { border: none; }

/* Links */
a:link, a:visited { background: transparent; font-weight: normal; text-decoration: underline; color:#333; }
a:link[href^="http://"]:after, a[href^="http://"]:visited:after { content: " (" attr(href) ")"; font-size: 90%; }
a[href^="http://"] {color:#000; }

/* Table */
table { margin: 1px; text-align:left; }
th { font-weight: bold; }
th,td { padding: 4px 10px 4px 0; }
tfoot { font-style: italic; }
caption { background: #fff; margin-bottom:2em; text-align:left; }
thead {display: table-header-group;}
tr {page-break-inside: avoid;} 

/*hide various parts from the site*/

#maincol { margin-left: 1em; margin-right: 1em; border: none; }
#content { border: none; }
#sidebar, #userbox, #footer {display:none;}
#toc { display: none; }
#categoryList { display: none; }
h1 a:link, h2 a:link, h3 a:link, h4 a:link, h5 a:link, h6 a:link { text-decoration: none; }
h1.pageTitle { font-size: 220%; }
td.lineNumbers { display: none; }
ul.tabs { display: none; }
